"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _fs = _interopRequireDefault(require("fs"));

var _chalk = _interopRequireDefault(require("chalk"));

var _webpack = _interopRequireDefault(require("webpack"));

var _commander = _interopRequireDefault(require("commander"));

var _clearConsole = _interopRequireDefault(require("react-dev-utils/clearConsole"));

var _FileSizeReporter = require("react-dev-utils/FileSizeReporter");

var _formatWebpackMessages = _interopRequireDefault(require("react-dev-utils/formatWebpackMessages"));

var _logger = _interopRequireDefault(require("../utils/logger"));

var _createConfig = _interopRequireDefault(require("../webpack/config/createConfig"));

var _printErrors = _interopRequireDefault(require("../utils/printErrors"));

// process.on('unhandledRejection', (reason, p) => {
//   console.log('Unhandled Rejection at:', p, 'reason:', reason);
// });
process.noDeprecation = true; // 关闭告警信息，避免对进度条显示产生干扰

_commander["default"].option('-v, --verbose', '显示详细日志信息').option('-V, --visualizer', '启用 webpack-visualizer 打包分析工具').parse(process.argv);

var _process$env = process.env,
    DACE_PATH_CONFIG = _process$env.DACE_PATH_CONFIG,
    DACE_PATH_CLIENT_DIST = _process$env.DACE_PATH_CLIENT_DIST; // 捕捉 webpack 编译过程中的错误

function compile(config, cb) {
  var compiler;

  try {
    compiler = (0, _webpack["default"])(config);
  } catch (e) {
    (0, _printErrors["default"])('Failed to compile.', [e]);
    process.exit(1);
  }

  compiler.run(function (err, stats) {
    cb(err, stats);
  });
}

function build(previousFileSizes) {
  var dace = {};

  if (_fs["default"].existsSync(DACE_PATH_CONFIG)) {
    try {
      dace = require(DACE_PATH_CONFIG);
    } catch (e) {
      (0, _clearConsole["default"])();

      _logger["default"].error("Invalid dace.config.js file. ".concat(e));

      process.exit(1);
    }
  }

  var clientConfig = (0, _createConfig["default"])({
    webpack: _webpack["default"],
    dace: dace,
    target: 'web',
    isDev: false,
    program: _commander["default"]
  });
  var serverConfig = (0, _createConfig["default"])({
    webpack: _webpack["default"],
    dace: dace,
    target: 'node',
    isDev: false,
    program: _commander["default"]
  });
  console.log('Creating an optimized production build...');

  if (_commander["default"].verbose) {
    console.log('Client build config:');
    console.dir(clientConfig, {
      showHidden: true,
      depth: 10
    });
  }

  console.log('Compiling client...');
  return new Promise(function (resolve, reject) {
    compile(clientConfig, function (clientError, clientStats) {
      // eslint-disable-line
      if (clientError) {
        reject(clientError);
      }

      var clientMessages = (0, _formatWebpackMessages["default"])(clientStats.toJson({}, true));

      if (clientMessages.errors.length) {
        return reject(new Error(clientMessages.errors.join('\n\n')));
      }

      console.log(_chalk["default"].green('Compiled client successfully.'));

      if (_commander["default"].visualizer) {
        var file = "".concat(DACE_PATH_CLIENT_DIST, "/stats.html");
        var message = "`webpack visualizer` has been generated.\nOpen it ".concat(_chalk["default"].underline("open file://".concat(file)));

        _logger["default"].info(message);
      }

      if (_commander["default"].verbose) {
        console.log('Server build config:');
        console.dir(serverConfig, {
          showHidden: true,
          depth: 10
        });
      }

      console.log('Compiling server...');
      compile(serverConfig, function (serverError, serverStats) {
        if (serverError) {
          reject(serverError);
        }

        var serverMessages = (0, _formatWebpackMessages["default"])(serverStats.toJson({}, true));

        if (serverMessages.errors.length) {
          return reject(new Error(serverMessages.errors.join('\n\n')));
        }

        console.log(_chalk["default"].green('Compiled server successfully.'));
        return resolve({
          stats: clientStats,
          previousFileSizes: previousFileSizes,
          warnings: Object.assign({}, clientMessages.warnings, serverMessages.warnings)
        });
      });
    });
  });
}

(0, _FileSizeReporter.measureFileSizesBeforeBuild)(DACE_PATH_CLIENT_DIST).then(function (previousFileSizes) {
  return build(previousFileSizes);
}).then(function (_ref) {
  var stats = _ref.stats,
      previousFileSizes = _ref.previousFileSizes,
      warnings = _ref.warnings;

  if (warnings.length) {
    console.log(_chalk["default"].yellow('Compiled with warnings.\n'));
    console.log(warnings.join('\n\n'));
    console.log("\nSearch for the ".concat(_chalk["default"].underline(_chalk["default"].yellow('keywords')), " to learn more about each warning."));
    console.log("To ignore, add ".concat(_chalk["default"].cyan('// eslint-disable-next-line'), " to the line before.\n"));
  } else {
    console.log(_chalk["default"].green('Compiled successfully.\n'));
  }

  console.log('File sizes after gzip:\n');
  (0, _FileSizeReporter.printFileSizesAfterBuild)(stats, previousFileSizes, DACE_PATH_CLIENT_DIST);
  console.log();
}, function (err) {
  console.log(_chalk["default"].red('Failed to compile.\n'));
  console.log("".concat(err.message || err, "\n"));
  process.exit(1);
});