"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _fs = _interopRequireDefault(require("fs"));

var _commander = _interopRequireDefault(require("commander"));

var _chalk = _interopRequireDefault(require("chalk"));

var _webpack = _interopRequireDefault(require("webpack"));

var _webpackDevServerSpeedy = _interopRequireDefault(require("webpack-dev-server-speedy"));

var _clearConsole = _interopRequireDefault(require("react-dev-utils/clearConsole"));

var _logger = _interopRequireDefault(require("../utils/logger"));

var _createConfig = _interopRequireDefault(require("../webpack/config/createConfig"));

var _setPorts = _interopRequireDefault(require("../utils/setPorts"));

_commander["default"].option('-s, --silent', '禁用所有输出信息').option('-v, --verbose', '显示详细日志信息').option('-V, --visualizer', '启用 webpack-visualizer 打包分析工具').parse(process.argv);

process.noDeprecation = true; // 关闭告警信息，避免对进度条显示产生干扰

process.env.NODE_ENV = 'local'; // 捕获 webpack 执行过程中的错误

function compile(config) {
  var compiler;

  try {
    compiler = (0, _webpack["default"])(config);
  } catch (e) {
    _logger["default"].error("Failed to compile: ".concat(e));

    process.exit(1);
  }

  return compiler;
}

function main() {
  var _process$env = process.env,
      DACE_PATH_CONFIG = _process$env.DACE_PATH_CONFIG,
      DACE_PATH_CLIENT_DIST = _process$env.DACE_PATH_CLIENT_DIST;
  var dace = {};

  if (_fs["default"].existsSync(DACE_PATH_CONFIG)) {
    try {
      dace = require(DACE_PATH_CONFIG);
    } catch (e) {
      (0, _clearConsole["default"])();

      _logger["default"].error('Invalid dace.config.js file.', e);

      process.exit(1);
    }
  }

  var clientConfig = (0, _createConfig["default"])({
    webpack: _webpack["default"],
    dace: dace,
    target: 'web',
    isDev: true,
    program: _commander["default"]
  });
  var serverConfig = (0, _createConfig["default"])({
    webpack: _webpack["default"],
    dace: dace,
    target: 'node',
    isDev: true,
    program: _commander["default"]
  });

  _logger["default"].start('Compiling...'); // Compile our assets with webpack


  var clientCompiler = compile(clientConfig);
  var serverCompiler = compile(serverConfig); // 在确保浏览器端编译成功后再启动服务器端编译

  clientCompiler.plugin('done', function (stats) {
    if (stats.compilation.errors.length === 0 && stats.compilation.warnings.length === 0) {
      if (_commander["default"].visualizer) {
        var file = "".concat(DACE_PATH_CLIENT_DIST, "/stats.html");
        var message = "`webpack visualizer` has been generated.\nOpen it ".concat(_chalk["default"].underline("open file://".concat(file)));

        _logger["default"].info(message);
      }

      serverCompiler.watch({
        quiet: true,
        stats: 'none'
      }, function () {});
    } else {
      _logger["default"].error('Client build failed.');
    }
  }); // Create a new instance of Webpack-dev-server for our client assets.
  // This will actually run on a different port than the users app.

  var clientDevServer = new _webpackDevServerSpeedy["default"](clientCompiler, clientConfig.devServer); // Start Webpack-dev-server

  var devPort = process.env.DACE_PORT && parseInt(process.env.DACE_PORT, 10) + 1 || 3001;
  clientDevServer.listen(devPort, function (err) {
    if (err) {
      _logger["default"].error(err);
    }
  });
} // 确保服务端口可用


(0, _setPorts["default"])().then(main)["catch"](console.error);