"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _fs = _interopRequireDefault(require("fs"));

var _logger = _interopRequireDefault(require("./logger"));

var _defaultEnv = _interopRequireDefault(require("./defaultEnv"));

var _resolveApp = _interopRequireDefault(require("./resolveApp"));

/**
 * 支持的环境变量如下：
 * NODE_ENV: NODE_ENV，由编译系统传入
 * PROFILE: 编译系统分组编号，由编译系统传入
 * DACE_PATH_ROOT: 工程根目录位置，测试可能用到
 * DACE_PATH_PROFILES: `profiles` 目录的绝对路径位置
 */
// 不存在 NODE_ENV 时为其设置默认值 `local`
if (!('NODE_ENV' in process.env)) {
  _logger["default"].warn('The NODE_ENV environment variable is required but was not specified.');

  process.env.NODE_ENV = 'local';
}

var _process$env = process.env,
    NODE_ENV = _process$env.NODE_ENV,
    PROFILE = _process$env.PROFILE,
    _process$env$DACE_PAT = _process$env.DACE_PATH_PROFILES,
    DACE_PATH_PROFILES = _process$env$DACE_PAT === void 0 ? _defaultEnv["default"].DACE_PATH_PROFILES : _process$env$DACE_PAT,
    _process$env$DACE_PAT2 = _process$env.DACE_PATH_CONFIG,
    DACE_PATH_CONFIG = _process$env$DACE_PAT2 === void 0 ? 'dace.config.js' : _process$env$DACE_PAT2;

_logger["default"].info("process.env.NODE_ENV: ".concat(NODE_ENV)); // 获取 proflies 目录的绝对路径


var profilesDir = (0, _resolveApp["default"])(DACE_PATH_PROFILES); // https://github.com/bkeepers/dotenv#what-other-env-files-can-i-use

var dotenvFiles = ["".concat(profilesDir, "/.").concat(PROFILE, ".env"), "".concat(profilesDir, "/.").concat(NODE_ENV, ".env"), "".concat(profilesDir, "/.common.env")]; // Load environment variables from .env* files. Suppress warnings using silent
// if this file is missing. dotenv will never modify any environment variables
// that have already been set.
// https://github.com/motdotla/dotenv

dotenvFiles.forEach(function (dotenvFile) {
  if (_fs["default"].existsSync(dotenvFile)) {
    require('dotenv').config({
      path: dotenvFile
    });
  }
}); // 读 dace.config.js 中的配置

var daceConfigFile = (0, _resolveApp["default"])(DACE_PATH_CONFIG);
var daceConfig = _fs["default"].existsSync(daceConfigFile) ? require(daceConfigFile) : {}; // `plugins` `modify` 不加入环境变量

var excludeKeys = ['plugins', 'modify'];
Object.keys(daceConfig).filter(function (key) {
  return excludeKeys.indexOf(key) === -1;
}).forEach(function (key) {
  if (!(key in process.env)) {
    process.env[key] = daceConfig[key];
  }
}); // 最后使用 dace 默认 dotenv 兜底

Object.keys(_defaultEnv["default"]).forEach(function (key) {
  if (!(key in process.env)) {
    process.env[key] = _defaultEnv["default"][key];
  }
}); // 完成所有环境变量设置后，将 DACE_PATH_ 开头的环境变量转换为绝对路径
// 目的是为了 webpack 编译时能通过 require(process.env.XX) 引用到对应的文件

Object.keys(process.env).filter(function (key) {
  return key.startsWith('DACE_PATH_');
}).forEach(function (key) {
  process.env[key] = (0, _resolveApp["default"])(process.env[key]);
});