"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _chalk = _interopRequireDefault(require("chalk"));

var logTypes = {
  warn: {
    bg: 'bgYellow',
    msg: ' WARNING ',
    text: 'yellow'
  },
  debug: {
    bg: 'bgMagenta',
    msg: ' DEBUG ',
    text: 'magenta'
  },
  info: {
    bg: 'bgCyan',
    msg: ' INFO ',
    text: 'cyan'
  },
  error: {
    bg: 'bgRed',
    msg: ' ERROR ',
    text: 'red'
  },
  start: {
    bg: 'bgBlue',
    msg: ' WAIT ',
    text: 'blue'
  },
  done: {
    bg: 'bgGreen',
    msg: ' DONE ',
    text: 'green'
  }
};

var write = function write(type, text, verbose) {
  var textToLog = '';
  var logObject = false;
  var logType = logTypes[type];
  textToLog = "".concat(textToLog).concat(_chalk["default"][logType.bg].black(logType.msg), " ").concat(_chalk["default"][logType.text](text)); // Adds optional verbose output

  if (verbose) {
    if ((0, _typeof2["default"])(verbose) === 'object') {
      logObject = true;
    } else {
      textToLog += "\n\n".concat(verbose);
    }
  }

  console.log(textToLog);

  if (['start', 'done', 'error'].indexOf(type) > -1) {
    console.log();
  }

  if (logObject) console.dir(verbose, {
    depth: 15
  });
}; // Printing any statements


var log = function log() {
  var text = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
  return console.log(text);
}; // Starting a process


var start = function start(text) {
  return write('start', text);
}; // Ending a process


var done = function done(text) {
  return write('done', text);
}; // Info about a process task


var info = function info(text) {
  return write('info', text);
}; // Verbose output
// takes optional data


var debug = function debug(text, data) {
  write('debug', text, data);
}; // Warn output


var warn = function warn(text, data) {
  write('warn', text, data);
}; // Error output
// takes an optional error


var error = function error(text, err) {
  write('error', text, err);
};

module.exports = {
  log: log,
  info: info,
  debug: debug,
  warn: warn,
  error: error,
  start: start,
  done: done
};