"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _util = _interopRequireDefault(require("util"));

var _webpackbar = _interopRequireDefault(require("webpackbar"));

var _cleanWebpackPlugin = require("clean-webpack-plugin");

var _startServerWebpackPlugin = _interopRequireDefault(require("start-server-webpack-plugin"));

var _errorOverlayMiddleware = _interopRequireDefault(require("react-dev-utils/errorOverlayMiddleware"));

var _eslintFormatter = _interopRequireDefault(require("react-dev-utils/eslintFormatter"));

var _webpackNodeExternals = _interopRequireDefault(require("webpack-node-externals"));

var _miniCssExtractPlugin = _interopRequireDefault(require("mini-css-extract-plugin"));

var _webpackVisualizerPlugin = _interopRequireDefault(require("webpack-visualizer-plugin"));

var _webpackPlugin = _interopRequireDefault(require("@loadable/webpack-plugin"));

var _stylelintFormatter = _interopRequireDefault(require("../../utils/stylelintFormatter"));

var _logger = _interopRequireDefault(require("../../utils/logger"));

/* eslint no-nested-ternary: 0 */

/**
 * webpack config 生成器
 *
 * @param {object} option
 * @param {object} option.webpack 父模块创建的 webpack 实例
 * @param {object} option.dace dace.config.js
 * @param {string} option.[target='web']
 * @param {boolean} option.[isDev=true] 是否为开发环境
 * @param {object} option.[program={}] 程序运行句柄，能获取到命令参数
 * @return {object} webpack 配置对象
 */
var _default = function _default(_ref) {
  var webpack = _ref.webpack,
      _ref$dace = _ref.dace,
      modify = _ref$dace.modify,
      plugins = _ref$dace.plugins,
      _ref$target = _ref.target,
      target = _ref$target === void 0 ? 'web' : _ref$target,
      _ref$isDev = _ref.isDev,
      isDev = _ref$isDev === void 0 ? true : _ref$isDev,
      _ref$program = _ref.program,
      program = _ref$program === void 0 ? {} : _ref$program;
  var _process$env = process.env,
      DACE_HOST = _process$env.DACE_HOST,
      DACE_PORT = _process$env.DACE_PORT,
      _process$env$NODE_PAT = _process$env.NODE_PATH,
      NODE_PATH = _process$env$NODE_PAT === void 0 ? '' : _process$env$NODE_PAT,
      DACE_PUBLIC_PATH = _process$env.DACE_PUBLIC_PATH,
      DACE_VENDORS = _process$env.DACE_VENDORS,
      DACE_LONG_TERM_CACHING = _process$env.DACE_LONG_TERM_CACHING,
      DACE_LONG_TERM_CACHING_LENGTH = _process$env.DACE_LONG_TERM_CACHING_LENGTH,
      DACE_PATH_BABEL_RC = _process$env.DACE_PATH_BABEL_RC,
      DACE_PATH_ESLINT_RC = _process$env.DACE_PATH_ESLINT_RC,
      DACE_PATH_POSTCSS_RC = _process$env.DACE_PATH_POSTCSS_RC,
      DACE_PATH_NODE_MODULES = _process$env.DACE_PATH_NODE_MODULES,
      DACE_PATH_CLIENT_ENTRY = _process$env.DACE_PATH_CLIENT_ENTRY,
      DACE_PATH_SERVER_ENTRY = _process$env.DACE_PATH_SERVER_ENTRY,
      DACE_PATH_CLIENT_DIST = _process$env.DACE_PATH_CLIENT_DIST,
      DACE_PATH_SERVER_DIST = _process$env.DACE_PATH_SERVER_DIST;
  var IS_NODE = target === 'node';
  var IS_WEB = target === 'web';
  var IS_DEV = isDev;
  var devServerPort = parseInt(DACE_PORT, 10) + 1;

  var getHash = function getHash(hash) {
    if (DACE_LONG_TERM_CACHING === 'true') {
      return ".[".concat(hash, ":").concat(DACE_LONG_TERM_CACHING_LENGTH, "]");
    }

    return '';
  }; // 将 process.env 中所有以 DACE_ 开头的变量传递到代码运行时环境


  var daceEnv = Object.keys(process.env).filter(function (key) {
    return key.startsWith('DACE_');
  }).reduce(function (envs, key) {
    envs["process.env.".concat(key)] = JSON.stringify(process.env[key]);
    return envs;
  }, {}); // 获取 .babelrc 配置

  var mainBabelOptions = {
    cacheDirectory: true,
    babelrc: false
  };

  var hasBabelRc = _fs["default"].existsSync(DACE_PATH_BABEL_RC);

  if (hasBabelRc) {
    if (IS_WEB) {
      _logger["default"].info('Using custom .babelrc');
    }

    mainBabelOptions = (0, _objectSpread2["default"])({}, mainBabelOptions, {
      configFile: DACE_PATH_BABEL_RC
    });
  } else {
    mainBabelOptions = (0, _objectSpread2["default"])({}, mainBabelOptions, {
      presets: [require.resolve('babel-preset-dace')]
    });
  } // 获取 .eslintrc.js 配置


  var hasEslintRc = _fs["default"].existsSync(DACE_PATH_ESLINT_RC);

  var mainEslintOptions = {
    formatter: _eslintFormatter["default"],
    // failOnError: true,
    eslintPath: require.resolve('eslint'),
    useEslintrc: true
  };

  if (hasEslintRc) {
    if (IS_WEB) {
      _logger["default"].info('Using custom .eslintrc.js');
    }
  } else {
    mainEslintOptions.configFile = _path["default"].resolve(__dirname, '../../../.eslintrc.js');
  } // 获取 postcss 配置


  var hasPostcssRc = _fs["default"].existsSync(DACE_PATH_POSTCSS_RC);

  var mainPostcssOptions = {
    ident: 'postcss'
  };

  if (hasPostcssRc) {
    if (IS_WEB) {
      _logger["default"].info('Using custom postcss.config.js');
    } // 只能指定 postcss.config.js 所在的目录


    mainPostcssOptions.config = {
      path: _path["default"].dirname(DACE_PATH_POSTCSS_RC)
    };
  } else {
    mainPostcssOptions.plugins = [require('stylelint')({
      config: {
        "extends": 'stylelint-config-dace'
      }
    }), require('postcss-preset-env')(), require('postcss-reporter')({
      throwError: true,
      clearReportedMessages: true,
      clearAllMessages: true,
      plugins: ['stylelint'],
      formatter: _stylelintFormatter["default"]
    })];
  }

  var config = {
    mode: IS_DEV ? 'development' : 'production',
    context: process.cwd(),
    target: target,
    devtool: 'cheap-module-source-map',
    resolve: {
      modules: ['node_modules', DACE_PATH_NODE_MODULES].concat(NODE_PATH.split(_path["default"].delimiter).filter(Boolean)),
      extensions: ['.js', '.jsx'],
      alias: {
        // This is required so symlinks work during development.
        'webpack/hot/poll': require.resolve('webpack/hot/poll')
      }
    },
    module: {
      strictExportPresence: true,
      rules: [{
        test: /\/ssrRoutes/,
        // 多个 loaders 按 `从后往前` 的顺序执行
        use: [{
          loader: require.resolve('babel-loader'),
          options: mainBabelOptions
        }, {
          loader: _path["default"].resolve(__dirname, '../loaders/routesLoader.js')
        }]
      }, {
        test: /\.(js|jsx)$/,
        // 强制在其他 loader 之前执行
        enforce: 'pre',
        // 本地调试时 dace 并未在 node_modules 目录下
        // 需要单独排除
        exclude: [/node_modules/, /\/dace[^/]*\/dist\//],
        use: [{
          options: mainEslintOptions,
          loader: require.resolve('eslint-loader')
        }]
      }, {
        test: /\.(js|jsx)$/,
        exclude: [/node_modules/],
        use: [{
          loader: require.resolve('babel-loader'),
          options: mainBabelOptions
        }]
      }, {
        exclude: [/\.html$/, /\.(js|jsx|mjs)$/, /\.(ts|tsx)$/, /\.(vue)$/, /\.(less)$/, /\.(re)$/, /\.(s?css|sass)$/, /\.json$/, /\.bmp$/, /\.gif$/, /\.jpe?g$/, /\.png$/],
        loader: require.resolve('file-loader'),
        options: {
          name: "media/[name]".concat(getHash('hash'), ".[ext]"),
          emitFile: true
        }
      }, // 'url' loader works like 'file' loader except that it embeds assets
      // smaller than specified limit in bytes as data URLs to avoid requests.
      // A missing `test` is equivalent to a match.
      {
        test: [/\.bmp$/, /\.gif$/, /\.jpe?g$/, /\.png$/],
        loader: require.resolve('url-loader'),
        options: {
          limit: 10000,
          name: "media/[name]".concat(getHash('hash'), ".[ext]"),
          emitFile: true
        }
      }, // 'postcss' loader applies autoprefixer to our CSS.
      // 'css' loader resolves paths in CSS and adds assets as dependencies.
      // 'style' loader turns CSS into JS modules that inject <style> tags.
      // In production, we use a plugin to extract that CSS to a file, but
      // in development 'style' loader enables hot editing of CSS.
      //
      // Note: this yields the exact same CSS config as create-react-app.
      {
        test: /\.css$/,
        exclude: [/\.module\.css$/],
        use: IS_NODE ? [{
          loader: require.resolve('css-loader'),
          options: {
            importLoaders: 1
          }
        }] : IS_DEV ? [require.resolve('style-loader'), {
          loader: require.resolve('css-loader'),
          options: {
            importLoaders: 1
          }
        }, {
          loader: require.resolve('postcss-loader'),
          options: mainPostcssOptions
        }] : [_miniCssExtractPlugin["default"].loader, {
          loader: require.resolve('css-loader'),
          options: {
            importLoaders: 1,
            modules: false
          }
        }, {
          loader: require.resolve('postcss-loader'),
          options: mainPostcssOptions
        }]
      }, // 支持 [CSS Modules](https://github.com/css-modules/css-modules)
      // 使用 `.module.css` 后缀
      {
        test: /\.module\.css$/,
        exclude: [DACE_PATH_CLIENT_DIST, DACE_PATH_SERVER_DIST],
        use: IS_NODE ? [{
          loader: require.resolve('css-loader'),
          options: {
            modules: {
              localIdentName: '[path]__[name]___[local]'
            },
            importLoaders: 1,
            // 服务器端编译不需要内联 css ，只需要获取混淆后的 class 名称
            onlyLocals: true
          }
        }] : IS_DEV ? [require.resolve('style-loader'), {
          loader: require.resolve('css-loader'),
          options: {
            modules: {
              localIdentName: '[path]__[name]___[local]'
            },
            importLoaders: 1
          }
        }, {
          loader: require.resolve('postcss-loader'),
          options: mainPostcssOptions
        }] : [_miniCssExtractPlugin["default"].loader, {
          loader: require.resolve('css-loader'),
          options: {
            modules: {
              localIdentName: '[path]__[name]___[local]'
            },
            importLoaders: 1
          }
        }, {
          loader: require.resolve('postcss-loader'),
          options: mainPostcssOptions
        }]
      }]
    },
    plugins: []
  };

  if (IS_NODE) {
    // We want to uphold node's __filename, and __dirname.
    config.node = {
      __dirname: false,
      __filename: false
    };
    config.externals = [(0, _webpackNodeExternals["default"])({
      // 白名单中的包会包含在编译后的包里
      whitelist: [IS_DEV ? 'webpack/hot/poll?300' : null, /dace/, // <-- #13
      /\.(eot|woff|woff2|ttf|otf)$/, /\.(svg|png|jpg|jpeg|gif|ico)$/, /\.(mp4|mp3|ogg|swf|webp)$/, /\.(css|scss|sass|sss|less)$/].filter(Boolean)
    })];
    config.output = {
      path: DACE_PATH_SERVER_DIST,
      publicPath: IS_DEV ? "http://".concat(DACE_HOST, ":").concat(devServerPort, "/") : '/',
      filename: 'server.js',
      libraryTarget: 'commonjs2'
    };
    config.entry = [_fs["default"].existsSync(DACE_PATH_SERVER_ENTRY) ? DACE_PATH_SERVER_ENTRY : _path["default"].resolve(__dirname, '../../runtime/server.js')];
    config.plugins = [// 将定义环境变量传递到运行时环境
    new webpack.DefinePlugin(daceEnv), // 防止 node 编译时打成多个包
    new webpack.optimize.LimitChunkCountPlugin({
      maxChunks: 1
    })];

    if (IS_DEV) {
      config.watch = true;
      config.entry.unshift('webpack/hot/poll?300');
      config.plugins = [].concat((0, _toConsumableArray2["default"])(config.plugins), [new webpack.HotModuleReplacementPlugin(), // Supress errors to console (we use our own logger)
      // StartServerPlugin 会出 DeprecationWarning: Buffer()
      new _startServerWebpackPlugin["default"]({
        name: 'server.js'
      }), // 不监视编译输出目录，避免重新压缩死循环
      new webpack.WatchIgnorePlugin([DACE_PATH_CLIENT_DIST, DACE_PATH_SERVER_DIST])]);
    }
  }

  if (IS_WEB) {
    config.entry = [_fs["default"].existsSync(DACE_PATH_CLIENT_ENTRY) ? DACE_PATH_CLIENT_ENTRY : _path["default"].resolve(__dirname, '../../runtime/client.js')];
    config.output = {
      path: DACE_PATH_CLIENT_DIST,
      libraryTarget: 'var'
    };
    config.plugins = [].concat((0, _toConsumableArray2["default"])(config.plugins), [new _cleanWebpackPlugin.CleanWebpackPlugin({
      cleanOnceBeforeBuildPatterns: [DACE_PATH_CLIENT_DIST, DACE_PATH_SERVER_DIST]
    }), new webpack.DefinePlugin(daceEnv), new _webpackPlugin["default"]({
      writeToDisk: true,
      outputAsset: false
    })]);
    var vendorPattern = new RegExp("(".concat(DACE_VENDORS, ")"));
    config.optimization = {
      minimize: false,
      splitChunks: {
        cacheGroups: {
          // 禁用 cacheGroups(test/priority/reuseExistingChunk)默认配置
          "default": false,
          // 禁用 vendors
          vendors: false,
          // 打包 vendor.js
          vendor: {
            name: 'vendor',
            test: vendorPattern,
            chunks: 'all',
            enforce: true
          },
          // 打包 styles.css
          styles: {
            name: 'styles',
            test: /\.(css|less|scss)$/,
            chunks: 'all',
            minChunks: 2,
            priority: 10,
            reuseExistingChunk: true,
            enforce: true
          }
        }
      }
    };

    if (IS_DEV) {
      config.entry.unshift(require.resolve('../../utils/webpackHotDevClient')); // Configure our client bundles output. Not the public path is to 3001.

      config.output = (0, _objectSpread2["default"])({}, config.output, {
        publicPath: "http://".concat(DACE_HOST, ":").concat(devServerPort, "/"),
        pathinfo: true,
        filename: "js/bundle".concat(getHash('hash'), ".js"),
        chunkFilename: "js/[name]".concat(getHash('hash'), ".chunk.js"),
        devtoolModuleFilenameTemplate: function devtoolModuleFilenameTemplate(info) {
          return _path["default"].resolve(info.resourcePath).replace(/\\/g, '/');
        }
      }); // Configure webpack-dev-server to serve our client-side bundle from
      // http://${process.env.DACE_HOST}:3001

      config.devServer = {
        disableHostCheck: true,
        clientLogLevel: 'none',
        // Enable gzip compression of generated files.
        compress: true,
        // watchContentBase: true,
        headers: {
          'Access-Control-Allow-Origin': "http://".concat(DACE_HOST, ":").concat(DACE_PORT),
          'Access-Control-Allow-Credentials': true
        },
        host: DACE_HOST,
        hot: true,
        noInfo: !program.verbose,
        overlay: false,
        port: devServerPort,
        quiet: !program.verbose,
        // By default files from `contentBase` will not trigger a page reload.
        // Reportedly, this avoids CPU overload on some systems.
        // https://github.com/facebookincubator/create-react-app/issues/293
        watchOptions: {
          ignored: /node_modules/
        },
        before: function before(app) {
          // This lets us open files from the runtime error overlay.
          app.use((0, _errorOverlayMiddleware["default"])());
        }
      }; // Add client-only development plugins

      config.plugins = [].concat((0, _toConsumableArray2["default"])(config.plugins), [new webpack.HotModuleReplacementPlugin({
        multiStep: true
      })]);
    } else {
      // web-build
      config.output = (0, _objectSpread2["default"])({}, config.output, {
        publicPath: DACE_PUBLIC_PATH,
        filename: "js/bundle".concat(getHash('chunkhash'), ".js"),
        chunkFilename: "js/[name]".concat(getHash('chunkhash'), ".chunk.js")
      });
      config.plugins = [].concat((0, _toConsumableArray2["default"])(config.plugins), [// Extract our CSS into a files.
      new _miniCssExtractPlugin["default"]({
        filename: "css/[name]".concat(getHash('contenthash'), ".css"),
        // allChunks: true because we want all css to be included in the main
        // css bundle when doing code splitting to avoid FOUC:
        // https://github.com/facebook/create-react-app/issues/2415
        allChunks: true
      }), new webpack.HashedModuleIdsPlugin() // ,
      // new webpack.optimize.AggressiveMergingPlugin()
      ]);
      config.optimization = (0, _objectSpread2["default"])({}, config.optimization, {
        minimize: true
      });
    }
  }

  if (IS_DEV && !program.verbose) {
    config.plugins = [].concat((0, _toConsumableArray2["default"])(config.plugins), [new _webpackbar["default"]({
      color: target === 'web' ? '#f5a623' : '#9013fe',
      name: target === 'web' ? 'client' : 'server'
    })]);
  }

  if (program.visualizer) {
    config.plugins = [].concat((0, _toConsumableArray2["default"])(config.plugins), [new _webpackVisualizerPlugin["default"]()]);
  }

  if (modify) {
    config = modify(config, {
      target: target,
      isDev: IS_DEV
    }, webpack);
  } // 绑定 dace 插件


  if (plugins) {
    if (!Array.isArray(plugins)) {
      plugins = [plugins];
    }

    var pluginPrefix = 'dace-plugin-';
    plugins.forEach(function (name) {
      // 解析带参数的插件 [['redux', {}], 'mobx']
      var options = {};

      if (_util["default"].isArray(name)) {
        var opts = {};
        var _name = name;

        var _name2 = (0, _slicedToArray2["default"])(_name, 2);

        name = _name2[0];
        opts = _name2[1];
        options = (0, _objectSpread2["default"])({}, options, opts);
      } // 支持类似 babel preset 的语法


      var completePluginName = name.startsWith(pluginPrefix) ? name : "".concat(pluginPrefix).concat(name);
      var dacePlugin;

      try {
        dacePlugin = require("".concat(completePluginName, "/plugin"));

        if (dacePlugin["default"]) {
          dacePlugin = dacePlugin["default"];
        }
      } catch (e) {
        _logger["default"].error("Not found dace plugin: ".concat(completePluginName));

        throw e;
      }

      if (dacePlugin.modify && _util["default"].isFunction(dacePlugin.modify)) {
        config = dacePlugin.modify(config, {
          target: target,
          isDev: isDev
        }, webpack, options);
      }
    });
  }

  return config;
};

exports["default"] = _default;
module.exports = exports.default;