"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = loader;

var _fs = require("fs");

var _path = require("path");

var _glob = _interopRequireDefault(require("glob"));

/* eslint global-require: 0, import/no-dynamic-require: 0 */

/**
 * 该文件为 target='web' 提供文件系统操作能力
 * 当  target='web' 时，`require('./routes')` 返回的内容为该 loader 返回的内容
 */
// import loaderUtils from 'loader-utils';
var _process$env = process.env,
    DACE_INDEX = _process$env.DACE_INDEX,
    DACE_PATH_SRC = _process$env.DACE_PATH_SRC,
    DACE_PATH_PAGES = _process$env.DACE_PATH_PAGES;

function loader() {
  // <- 此次请勿使用箭头函数，否则会导致 this 不符合预期
  function getEndpointFromPath(pagePath) {
    var endpoint = "/".concat(pagePath);

    while (endpoint.endsWith("/".concat(DACE_INDEX))) {
      endpoint = endpoint.replace("/".concat(DACE_INDEX), '');
    }

    return endpoint || '/';
  }

  function getComponentPath(file) {
    var appComponent = (0, _path.resolve)(DACE_PATH_SRC, "components/".concat(file, ".js"));

    if ((0, _fs.existsSync)(appComponent)) {
      return appComponent;
    }

    return (0, _path.resolve)(__dirname, "../../runtime/components/".concat(file, ".js"));
  } // const options = loaderUtils.getOptions(this) || {};


  var pageExtension = '.jsx';

  var routes = _glob["default"].sync("**/*".concat(pageExtension), {
    cwd: DACE_PATH_PAGES
  }).map(function (item) {
    return item.replace(pageExtension, '');
  }).map(function (name) {
    var pathWithoutExtension = (0, _path.resolve)(DACE_PATH_PAGES, name);
    var endpoint;
    var exact; // 当存在路由文件时，页面的路由地址以路由文件中配置的 path 为准

    var routerFile = (0, _path.resolve)((0, _path.dirname)(pathWithoutExtension), 'router.js');

    if ((0, _fs.existsSync)(routerFile)) {
      var config = require(routerFile);

      endpoint = config.path;
      exact = config.exact || 'false';
    } else {
      endpoint = getEndpointFromPath(name);
      exact = (endpoint === '/').toString();
    }

    return "{\n        path: ".concat(JSON.stringify(endpoint), ",\n        exact: ").concat(exact, ",\n        component: loadable(() => import(/* webpackChunkName: \"").concat(name, "\" */ '").concat(pathWithoutExtension, "'))\n      }");
  }); // 在 routes 最后添加 404 找不到网页的路由


  routes.push("{\n    component: require('".concat(getComponentPath('NotFound'), "')\n  }"));
  return "\n    import loadable from '@loadable/component';\n    export default [\n      {\n        component: require('".concat(getComponentPath('App'), "'),\n        routes: [\n          ").concat(routes.join(','), "\n        ]\n      }\n    ];\n  ");
}

module.exports = exports.default;